<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Block\Frontend\LayoutProcessors;

use Magento\Checkout\Block\Checkout\LayoutProcessorInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Montonio\Shipping\Gateway\Shippings\Config\Config;

/**
 * @inheritDoc
 */
class MethodConfigProcessor implements LayoutProcessorInterface
{
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Config
     */
    protected $config;

    /**
     * Construct
     *
     * @param ScopeConfigInterface $scopeConfig
     * @param Config $config
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Config               $config
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->config = $config;
    }

    /**
     * Process the jslayout and add your things
     *
     * @param array $jsLayout
     * @return array
     */
    public function process($jsLayout)
    {

        //if not active don't react
        if (!$this->config->isActive()) {
            return $jsLayout;
        }

        $jsLayout['components']['checkout']
            ['children']['steps']
            ['children']['shipping-step']
            ['children']['step-config']
            ['children']['shipping-rates-validation']
            ['children']['montonio_shipping'] = [
                'component' => 'Montonio_Shipping/js/view/shipping'
            ];

            if (!empty($this->config->getShippingPoints())) {
                //provide config values for enabled  pickup points to checkoutconfig
                $structure = $this->getPointsAsOptroup($this->config->getShippingPoints()['points']);

                foreach ($structure as $key => $item) {
                    $jsLayout['components']['checkout']
                    ['children']['steps']
                    ['children']['shipping-step']
                    ['children']['step-config']
                    ['children']['shipping-rates-validation']
                    ['children']['montonio_shipping']
                    ['children'][$key] =
                    [
                        'component' => 'uiComponent',
                        'displayArea' => $key,
                        'children' => [
                            $key => [
                                'component' => 'Magento_Ui/js/form/element/select',
                                'config' => [
                                    'template' => 'ui/form/field',
                                    'elementTmpl' => 'ui/form/element/select',
                                    'options' => $item,
                                    'caption' => __('Pick your location'),
                                    'optionsCaption' => true,
                                    'provider' => 'checkoutProvider',
                                    'deps' => ['checkoutProvider'],
                                    'tracks' => ['value' => true],
                                    'statefull' => ['value' => true]
                                ],
                                'dataScope' => 'montonio_shipping.' . $key,
                                'validation' => [
                                    'required-entry' => true
                                ]
                            ]
                        ]
                    ];
                }

                $jsLayout['components']['checkout']
                ['children']['steps']
                ['children']['shipping-step']
                ['children']['step-config']
                ['children']['shipping-rates-validation']
                ['children']['montonio_shipping']['config']['points'] = array_keys($structure);

                $jsLayout['components']['checkout']
                ['children']['steps']
                ['children']['billing-step']
                ['children']['payment']
                ['children']['additional-payment-validators']
                ['children']['montonio_shipping'] = [
                    'component' => 'Montonio_Shipping/js/view/additional-validator'
                ];
            }

            return $jsLayout;
    }

    /**
     * Get pickupPoints as optgroup structure
     *
     * @param array $shippingPoints
     * @return array
     */
    protected function getPointsAsOptroup(array $shippingPoints): array
    {
        $structure = [];
        foreach ($shippingPoints as $point => $points) {
            $i = 0;
            foreach ($points as $pointLabel => $subPoints) {
                $structure[$point][$i] = [
                    'label' => $pointLabel,
                ];
                foreach ($subPoints as $subPointKey => $subPointLabel) {
                    $structure[$point][$i]['value'][] = [
                        'label' => $subPointLabel,
                        'value' => $subPointKey
                    ];
                }
                $i++;
            }
        }
        return $structure;
    }
}
