<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Adminhtml\Order;

use Exception;
use Montonio\Shipping\Gateway\Export\ShipmentExport;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Sales\Api\ShipmentRepositoryInterface;

class Export extends Action
{
    /**
     * @var ShipmentExport
     */
    protected $shipmentExport;

    /**
     * @var ShipmentRepositoryInterface
     */
    protected $shipmentRepository;

    /**
     * @var $resultRedirectFactory
     */
    protected $resultRedirectFactory;

    /**
     * @var $messageManager
     */
    protected $messageManager;

    /**
     * Construct function.
     *
     * @param Context $context
     * @param ShipmentExport $shipmentExport
     * @param ShipmentRepositoryInterface $shipmentRepository
     */
    public function __construct(
        Action\Context $context,
        ShipmentExport $shipmentExport,
        ShipmentRepositoryInterface $shipmentRepository
    ) {
        parent::__construct($context);
        $this->shipmentExport = $shipmentExport;
        $this->shipmentRepository = $shipmentRepository;
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
        $shipmentId = $this->getRequest()->getParam('shipment_id') ?? false;

        if ($shipmentId) {
            try {
                $shipment = $this->shipmentRepository->get((int)$shipmentId);

                //we are async here and not waiting for feedback
                $exportResult = $this->shipmentExport->export($shipment, false);

                $this->shipmentRepository->save($shipment);

                if ($exportResult !== false) {
                    $this->messageManager->addSuccessMessage(__('Shipment was successfully exported.'));
                } else {
                    $this->messageManager->addErrorMessage(
                        __('There was an error exporting shipment. Please, check logs.')
                    );
                }
            } catch (Exception $e) {
                $this->messageManager->addExceptionMessage(
                    $e,
                    __('Something went wrong while exporting shipment: %1', $e->getMessage())
                );
            }

            $resultRedirect = $this->resultRedirectFactory->create()->setPath('adminhtml/order_shipment/view/', [
                'shipment_id' => $shipmentId,
            ]);
        } else {
            $this->messageManager->addErrorMessage(__('Shipment not found.'));
            $resultRedirect = $this->resultRedirectFactory->create()->setPath('sales/order/index/');
        }

        return $resultRedirect;
    }
}
