<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Adminhtml\Order;

use Montonio\Shipping\Gateway\Export\PrintLabel;
use Exception;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Api\ShipmentRepositoryInterface;

class Label extends Action
{
    /**
     * @var PrintLabel $printLabel
     */
    protected $printLabel;

    /**
     * @var Context $context
     */
    protected $context;

    /**
     * @var ShipmentRepositoryInterface $shipmentRepository
     */
    protected $shipmentRepository;

    /**
     * @var $messageManager
     */
    protected $messageManager;

    /**
     * @var $resultRedirectFactory
     */
    protected $resultRedirectFactory;

    /**
     * Construct function.
     *
     * @param Context $context
     * @param PrintLabel $printLabel
     * @param ShipmentRepositoryInterface $shipmentRepository
     */
    public function __construct(
        Action\Context $context,
        PrintLabel $printLabel,
        ShipmentRepositoryInterface $shipmentRepository
    ) {
        parent::__construct($context);

        $this->printLabel = $printLabel;
        $this->shipmentRepository = $shipmentRepository;

        $this->context = $context;
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {

        $shipmentId = $this->getRequest()->getParam('shipment_id') ?? false;

        $returnUrl = $this->resultRedirectFactory->create()->setPath('adminhtml/order_shipment/view/', [
            'shipment_id' => (int)$shipmentId,
        ]);

        if ($shipmentId) {
            try {
                $shipment = $this->shipmentRepository->get((int)$shipmentId) ?? false;
                if ($shipment) {
                    $result = $this->printLabel->printLabel($shipment);
                    $this->messageManager->addSuccessMessage(__('Label was successfully printed.'));
                    if (isset($result->url)) {
                        $returnUrl = $this->resultRedirectFactory->create()->setPath($result->url);
                    }
                } else {
                    $this->messageManager->addErrorMessage(
                        __('A label printing error has occurred. Please, check logs.')
                    );
                }

            } catch (Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while printing the label.'));
            }
        }
        return $returnUrl;
    }
}
