<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Controller\Adminhtml\System\Config;

use Magento\Config\Controller\Adminhtml\System\Config\Save as SaveConfig;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\ProductMetadataInterface;
use Montonio\Shipping\Model\Shipping\ShippingsConfigProvider;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * @inheritDoc
 */
class Save extends SaveConfig
{

    /**
     * @var $_request
     */
    protected $_request;

    /**
     * @inheritDoc
     */
    public function execute()
    {
        //@TODO refactor out objectManager by replacing this with plugin
        $objectManager = ObjectManager::getInstance();
        $magentoVersion = $objectManager->get(ProductMetadataInterface::class);

        if (version_compare($magentoVersion->getVersion(), '2.4.0', '>=')) {
            return parent::execute();
        }
        //forced saving of configurations as it does not work in version 2.3.
        $params = $this->_request->getParam('groups');

        $code = $params['montonio_shipping'] ?? false;
        $availableMethods = $params['montonio_shipping']
                                    ['groups']
                                    ['montonio_methods']['fields']
                                    ['available_methods']['value'] ?? false;

        if ($code && $availableMethods) {
            if (isset($availableMethods['__empty'])) {
                unset($availableMethods['__empty']);
            }
            //@TODO refactor out objectManager
            $configWriter = $objectManager->create(WriterInterface::class);
            $path = sprintf(
                Config::DEFAULT_PATH_PATTERN,
                ShippingsConfigProvider::CODE,
                'montonio_methods/available_methods'
            );
            $configWriter->save(
                $path,
                json_encode($availableMethods),
                $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
                $scopeId = 0
            );
        }

        return parent::execute();
    }
}
