<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model;

use \Exception;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\ShipmentItemCreationInterfaceFactory;
use Magento\Sales\Api\Data\ShipmentCommentCreationInterfaceFactory;
use Magento\Sales\Api\ShipOrderInterface;

class CreateShipment
{
    /**
     * @const MONTONIO_SHIPPING_CREATED
     */
    public const MONTONIO_SHIPPING_CREATED = 'montonio_shipping_created';

    /**
     * @var ShipOrderInterface
     */
    protected $shipOrder;

    /**
     * @var ShipmentItemCreationInterfaceFactory
     */
    protected $shipmentItemCreationFactory;

    /**
     * @var ShipmentCommentCreationInterfaceFactory
     */
    protected $shipmentCommentCreationInterfaceFactory;

    /**
     * @inheritDoc
     */
    public function __construct(
        ShipOrderInterface $shipOrder,
        ShipmentItemCreationInterfaceFactory $shipmentItemCreationInterfaceFactory,
        ShipmentCommentCreationInterfaceFactory $shipmentCommentCreationInterfaceFactory
    ) {
        $this->shipOrder = $shipOrder;
        $this->shipmentItemCreationFactory = $shipmentItemCreationInterfaceFactory;
        $this->shipmentCommentCreationInterfaceFactory = $shipmentCommentCreationInterfaceFactory;
    }

    /**
     * Create shipment object
     *
     * @param OrderInterface $order
     * @param bool $flag
     * @param string $comment
     * @return void
     */
    public function execute(OrderInterface $order, bool $flag, string $comment = ''): void
    {

        try {
            $items = [];
            foreach ($order->getItems() as $orderItem) {
                $shipmentItemCreation = $this->shipmentItemCreationFactory->create();
                $shipmentItemCreation->setOrderItemId($orderItem->getItemId());
                $shipmentItemCreation->setQty($orderItem->getQtyOrdered());
                $items[] = $shipmentItemCreation;
                $shipmentItemCreation = null;
            }

            if (!empty($comment)) {
                $shipmentComment = $this->shipmentCommentCreationInterfaceFactory->create();
                $shipmentComment->setComment(__($comment));
                $comment = $shipmentComment;
            } else {
                $comment = null;
            }

            $this->shipOrder->execute(
                (int)$order->getEntityId(),
                $items,
                false,
                false,
                $comment,
                [],
                [],
                null
            );
            //special flag to track the order status later on status plugin
            if ($flag) {
                $order->setData(self::MONTONIO_SHIPPING_CREATED, 1)->save();
            }
            $items = [];
        } catch (Exception $e) {
            //unset the values if we have errors in our workflow
            if ($flag && $order->getData(self::MONTONIO_SHIPPING_CREATED) === 1) {
                $order->setData(self::MONTONIO_SHIPPING_CREATED, 0)->save();
            }
        }
    }
}
