<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model\Observer;

use Exception;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Model\Order;
use Montonio\Shipping\Gateway\Export\ShipmentExport;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Psr\Log\LoggerInterface;

class ExportShipmentSaveAfter implements ObserverInterface
{
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var ShipmentExport
     */
    protected $shipmentExport;

    /**
     * @inheritDoc
     */
    public function __construct(
        LoggerInterface $logger,
        Config $config,
        ShipmentExport $shipmentExport
    ) {
        $this->logger = $logger;
        $this->config = $config;
        $this->shipmentExport = $shipmentExport;
    }

    /**
     * @inheritDoc
     */
    public function execute(Observer $observer)
    {

        $shipment = $observer->getEvent()->getShipment();
        /** @var Order $order */
        $order = $shipment->getOrder();

        if ($this->config->matchShippingMethod($order->getShippingMethod())) {
            try {
                $this->shipmentExport->export($shipment, false);
            } catch (Exception $exception) {
                $this->logger->error('There was an error performing shipment export.', [
                    'exception_message' => $exception->getMessage(),
                    'shipment_increment_id' => $shipment->getIncrementId(),
                ]);
            }
        }
    }
}
