<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model\Observer;

use Magento\Framework\DataObject\Copy;
use Magento\Framework\Event\Observer as EventObserver;
use Magento\Framework\Event\ObserverInterface;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Model\ShippingPointsFactory;
use Montonio\Shipping\Model\Carrier\MontonioShipping;

class SaveQuoteDataToOrder implements ObserverInterface
{
    /**
     * @var ShippingPointsFactory
     */
    protected $shippingPointsFactory;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var Copy
     */
    private $objectCopyService;

    /**
     * @inheritDoc
     */
    public function __construct(
        Config $config,
        ShippingPointsFactory $shippingPointsFactory,
        Copy $objectCopyService
    ) {
        $this->shippingPointsFactory = $shippingPointsFactory;
        $this->config = $config;
        $this->objectCopyService = $objectCopyService;
    }

    /**
     * @inheritDoc
     */
    public function execute(EventObserver $observer)
    {

        /* @var Magento\Sales\Model\Order $order */
        $order = $observer->getEvent()->getData('order');
        /* @var Magento\Quote\Model\Quote $quote */
        $quote = $observer->getEvent()->getData('quote');

        $quote->setMontonioShippingPoint($quote->getBillingAddress()
                                                ->getExtensionAttributes()->getMontonioShippingPoint());

        $this->objectCopyService->copyFieldsetToTarget('sales_convert_quote', 'to_order', $quote, $order);

        $order = $observer->getOrder();
        $method = $quote->getShippingAddress()->getShippingMethod();

        if (strpos($method, MontonioShipping::CARRIER_CODE) !== false) {
            $point = $quote->getMontonioShippingPoint();
            $order->setCanSendNewEmailFlag(true);

            if ($point) {
                $montonioMethods = $this->config->getAvailableMethods('', $order->getStoreId());
                $parcelPoint = $this->shippingPointsFactory->create();
                $parcelPoint->load($point);
                $title = $parcelPoint->getShippingCode();

                $method = $montonioMethods[$parcelPoint->getShippingCode()] ?? false;
                if ($method) {
                    $title = $method['method_title'] . ' ' . $method['method_name'];
                }

                $order->setShippingDescription($title . ' - ' .$parcelPoint->getName());
            }
        }
        return $this;
    }
}
