<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Model;

use Magento\Framework\Model\AbstractModel;
use Magento\Framework\DataObject\IdentityInterface;
use Montonio\Shipping\Api\Data\ShippingPointsInterface;

class ShippingPoints extends AbstractModel implements ShippingPointsInterface, IdentityInterface
{
    /**
     * @var string
     */
    public const CACHE_TAG = 'montonio_shipping_points';

    /**
     * Constructor
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\ShippingPoints::class);
    }

    /**
     * Get Shipping Code
     *
     * @return string|null
     */
    public function getShippingCode()
    {
        return $this->getData(self::SHIPPING_CODE);
    }

    /**
     * Get Name
     *
     * @return string|null
     */
    public function getName()
    {
        return $this->getData(self::NAME);
    }

    /**
     * Get Region
     *
     * @return string|null
     */
    public function getRegion()
    {
        return $this->getData(self::REGION);
    }

    /**
     * Get Locality
     *
     * @return string|null
     */
    public function getLocality()
    {
        return $this->getData(self::LOCALITY);
    }

    /**
     * Get UUID
     *
     * @return string|null
     */
    public function getUuid()
    {
        return $this->getData(self::UUID);
    }

    /**
     * Get Identities
     *
     * @return array
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getUuid()];
    }

    /**
     * Set Shipping Code
     *
     * @param string $shippingCode
     * @return $this
     */
    public function setShippingCode(string $shippingCode)
    {
        return $this->setData(self::SHIPPING_CODE, $shippingCode);
    }

    /**
     * Set Name
     *
     * @param string $name
     * @return $this
     */
    public function setName(string $name)
    {
        return $this->setData(self::NAME, $name);
    }

    /**
     * Set Region
     *
     * @param string $region
     * @return $this
     */
    public function setRegion(string $region)
    {
        return $this->setData(self::REGION, $region);
    }

    /**
     * Set Locality
     *
     * @param string $locality
     * @return $this
     */
    public function setLocality(string $locality)
    {
        return $this->setData(self::LOCALITY, $locality);
    }

    /**
     * Set UUID
     *
     * @param string $uuid
     * @return $this
     */
    public function setUuid($uuid)
    {
        return $this->setData(self::UUID, $uuid);
    }
}
