<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Plugin\Adminhtml;

use Magento\Shipping\Block\Adminhtml\View;
use Magento\Sales\Api\ShipmentRepositoryInterface;
use Magento\Framework\App\RequestInterface;
use Montonio\Shipping\Gateway\Shippings\Config\Config;

class OrderViewPage
{

    /**
     * @var OrderRepositoryInterface
     */
    protected $shipmentRepository;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var Config
     */
    protected $config;

    /**
     * Construct function.
     *
     * @param ShipmentRepositoryInterface $shipmentRepository
     * @param RequestInterface $request
     * @param Config $config
     */
    public function __construct(
        ShipmentRepositoryInterface $shipmentRepository,
        RequestInterface $request,
        Config $config
    ) {
        $this->shipmentRepository = $shipmentRepository;
        $this->request = $request;
        $this->config = $config;
    }

    /**
     * Plugin for beforeSetLayout method
     *
     * @param $subject
     * @return void
     */
    public function beforeSetLayout($subject) : void
    {
        $shipmentId = $this->request->getParam('shipment_id') ?? false;

        if ($shipmentId) {

            $shipment = $this->shipmentRepository->get((int)$shipmentId);
            $order = $shipment->getOrder();

            //show buttons only form Montonio shippings
            if (!$this->config->matchShippingMethod($order->getShippingMethod())) {
                return;
            }

            $className = count($shipment->getAllTracks()) ? '' : 'primary action-secondary';

            $exportUrl = $subject->getUrl('montonio_shipping/order/export', ['shipment_id' => (int)$shipmentId]);
            $subject->addButton(
                'order_montonio_export',
                [
                    'label' => __('Export to Montonio Shipping'),
                    'class' => $className,
                    'onclick' => 'setLocation(\'' . $exportUrl . '\')'
                ]
            );

            if (count($shipment->getAllTracks()) > 0) {
                $printLabel = $subject->getUrl('montonio_shipping/order/label', ['shipment_id' => (int)$shipmentId]);
                $subject->addButton(
                    'order_montonio_print_label',
                    [
                        'label' => __('Print Montonio Shipping label'),
                        'class' => 'primary action-secondary',
                        'onclick' => 'setLocation(\'' . $printLabel . '\')'
                    ]
                );
            }

        }
    }
}
