<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Plugin\Shipping;

use Magento\Sales\Model\Order;
use Montonio\Shipping\Gateway\Shippings\Config\Config;

class AfterOrderSaveState
{
    /**
     * @var Config
     */
    protected $config;

    /**
     * @inheritDoc
     */
    public function __construct(
        Config $config
    ) {
        $this->config = $config;
    }

    /**
     * Before save on Order status
     *
     * @param \Magento\Sales\Model\ResourceModel\Order $subject
     * @param object $order
     * @return array
     */
    public function beforeSave(
        \Magento\Sales\Model\ResourceModel\Order $subject,
        $order
    ) {

        $pendingStatuses = ['pending', Order::STATE_PENDING_PAYMENT];

        //if pending and create shipment
        $isActive = (boolean)$this->config->getValue('active') ?? false;
        $createShipment = $this->config->getValue('create_shipment_objects', $order->getStoreId());
        $createShipmentsOnPending = in_array($createShipment, $pendingStatuses);

        if ($isActive &&
            $this->config->matchShippingMethod($order->getShippingMethod()) &&
            $createShipmentsOnPending &&
            $order->getData('montonio_shipping_created') === 1 &&
            !$order->canShip()
        ) {
            $order->setStatus('pending');
            $order->setState('new');
        }

        return [$order];
    }
}
