<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Plugin\Shipping;

use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order;
use Montonio\Shipping\Gateway\Shippings\Config\Config;
use Montonio\Shipping\Model\CreateShipment;
use Psr\Log\LoggerInterface;

/**
 * Plugin class for after save order event
 */
class AfterSaveOrder
{

    /**
     * @const MONTONIO_SHIPPING_CREATED
     */
    public const MONTONIO_SHIPPING_CREATED = 'montonio_shipping_created';

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var CreateShipment
     */
    protected $createShipment;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @inheritDoc
     */
    public function __construct(
        Config $config,
        CreateShipment $createShipment,
        LoggerInterface $logger
    ) {
        $this->config = $config;
        $this->createShipment = $createShipment;
        $this->logger = $logger;
    }

    /**
     * Plugin for afterSave on \Magento\Sales\Api\OrderRepositoryInterface
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderInterface $order
     * @return OrderInterface
     */
    public function afterSave(
        OrderRepositoryInterface $subject,
        OrderInterface $order
    ) {

        $pendingStatuses = ['pending', Order::STATE_PENDING_PAYMENT];
        $processingStatuses = [Order::STATE_PROCESSING, Order::STATE_PAYMENT_REVIEW];

        //if pending and create shipment
        $isActive = (boolean)$this->config->getValue('active') ?? false;
        $createShipment = $this->config->getValue('create_shipment_objects', $order->getStoreId());
        $createShipmentsOnPending = in_array($createShipment, $pendingStatuses);
        $createShipmentsOnProcessing = in_array($createShipment, $processingStatuses);
        $statusIsPending = in_array($order->getStatus(), $pendingStatuses);
        $statusIsProcessing = in_array($order->getStatus(), $processingStatuses);

        if (!$order->getIsVirtual() &&
            $isActive === true &&
            $this->config->matchShippingMethod($order->getShippingMethod()) &&
            $createShipmentsOnPending &&
            $statusIsPending &&
            $order->canShip()
        ) {
            $this->createShipment->execute($order, true, '');
        }

        if (!$order->getIsVirtual() &&
            $isActive === true &&
            $this->config->matchShippingMethod($order->getShippingMethod()) &&
            $createShipmentsOnProcessing &&
            $statusIsProcessing &&
            $order->canShip()
        ) {
            $this->createShipment->execute($order, true, '');
        }

        return $order;
    }
}
