<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

declare(strict_types=1);

namespace Montonio\Shipping\Plugin\Shipping;

use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\QuoteManagement;
use Montonio\Shipping\Model\Carrier\MontonioShipping;

class ValidateLocationExists
{

    /**
     * Before submitOrder plugin.
     *
     * @param QuoteManagement $subject
     * @param Quote $quote
     * @param array $orderData
     * @return void
     * @throws LocalizedException
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function beforeSubmit(
        QuoteManagement $subject,
        Quote           $quote,
        array           $orderData = []
    ): void {

        $method = $quote->getShippingAddress()->getShippingMethod() ?? false;
        $location = $quote->getBillingAddress()->getExtensionAttributes()->getMontonioShippingPoint() ?? false;

        if ($method &&
            strpos($method, MontonioShipping::CARRIER_CODE) === 0 &&
            strpos($method, MontonioShipping::COURIER_SUFFIX) === false &&
            !$location
        ) {
            throw new LocalizedException(__('Please pick the location for shipping method %1', $method));
        }
    }
}
