<?php
/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace Montonio\Shipping\Setup\Patch\Data;

use Magento\Catalog\Model\Product;
use Magento\Eav\Model\Entity\Attribute\Backend\ArrayBackend;
use Magento\Eav\Model\Entity\Attribute\FrontendLabelFactory;
use Magento\Catalog\Model\Product\Attribute\Repository;
use Magento\Eav\Model\Entity\Attribute\Source\Table;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Psr\Log\LoggerInterface;

class InstallAttribute implements DataPatchInterface
{
    /** @const string  */
    public const ATTRIBUTE_CODE = 'nomontonioshipping';

    /** @var EavSetupFactory  */
    protected $eavSetupFactory;

    /** @var ModuleDataSetupInterface  */
    protected $dataSetup;

    /** @var LoggerInterface  */
    protected $logger;

    /** @var Repository  */
    protected $attributeRepository;

    /**
     *
     * @param Repository $attributeRepository
     * @param EavSetupFactory $eavSetupFactory
     * @param ModuleDataSetupInterface $dataSetup
     * @param LoggerInterface $logger
     */
    public function __construct(
        Repository $attributeRepository,
        EavSetupFactory $eavSetupFactory,
        ModuleDataSetupInterface $dataSetup,
        LoggerInterface $logger
    ) {
        $this->attributeRepository = $attributeRepository;
        $this->eavSetupFactory = $eavSetupFactory;
        $this->dataSetup = $dataSetup;
        $this->logger = $logger;
    }

    /**
     * @inheritdoc
     */
    public function apply(): self
    {
        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->dataSetup]);

        try {

            $attr = $eavSetup->getAttribute(Product::ENTITY, self::ATTRIBUTE_CODE);

            if (!empty($attr['attribute_id'])) {
                //$eavSetup->removeAttribute(Product::ENTITY, self::ATTRIBUTE_CODE);
                return $this;
            }

            $options = [
                'EE_omniva_post_office',
                'EE_omniva_parcel_machine',
                'EE_omniva_courier',
                'EE_dpd_parcel_machine',
                'EE_dpd_courier',
                'EE_itella_parcel_machine',
                'EE_itella_courier',
                'FI_dpd_parcel_machine',
                'FI_dpd_courier',
                'FI_itella_parcel_machine',
                'FI_itella_post_office',
                'FI_itella_courier',
                'LT_omniva_parcel_machine',
                'LT_omniva_courier',
                'LT_dpd_parcel_machine',
                'LT_dpd_courier',
                'LV_omniva_parcel_machine',
                'LV_omniva_courier',
                'LV_dpd_parcel_machine',
                'LV_dpd_courier'
            ];

            $optionsData = [];

            foreach ($options as $option) {
                $optionsData[$option] = [$option];
            }

            $eavSetup->addAttribute(
                Product::ENTITY,
                self::ATTRIBUTE_CODE,
                [
                    'attribute_model' => null,
                    'frontend_class' => null,
                    'note' => null,
                    'frontend_input_renderer' => null,
                    'is_html_allowed_on_front' => '0',
                    'is_used_for_price_rules' => '0',
                    'used_in_product_listing' => '1',
                    'used_for_sort_by' => '0',
                    'apply_to' => null,
                    'position' => '0',
                    'is_required_in_admin_store' => '0',
                    'is_used_in_grid' => '0',
                    'is_visible_in_grid' => '0',
                    'is_filterable_in_grid' => '0',
                    'search_weight' => '1',
                    'is_pagebuilder_enabled' => '0',
                    'backend' => ArrayBackend::class,
                    'type' => 'text',
                    'table' => null,
                    'frontend' => null,
                    'input' => 'multiselect',
                    'label' => 'Montonio shipping terminals denied',
                    'source' => Table::class,
                    'required' => '0',
                    'user_defined' => '1',
                    'default' => null,
                    'unique' => '0',
                    'global' => '1',
                    'visible' => '1',
                    'searchable' => '0',
                    'filterable' => '0',
                    'comparable' => '0',
                    'visible_on_front' => '0',
                    'wysiwyg_enabled' => '0',
                    'visible_in_advanced_search' => '0',
                    'filterable_in_search' => '0',
                    'used_for_promo_rules' => '0',
                    'option' => ['value' => $optionsData ]
                ]
            );
        } catch (LocalizedException $e) {
            $this->logger->error($e->getMessage());
        }

        return $this;
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies(): array
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public function getAliases(): array
    {
        return [];
    }
}
