/**
 * montonio/module-shipping
 *
 * @category   montonio
 * @package    module-shipping
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
define(
    [
        "uiComponent",
        "uiRegistry",
        "ko",
        "underscore",
        "jquery",
        "mage/translate",
        "mage/utils/wrapper",
        "Magento_Ui/js/lib/view/utils/dom-observer",
        "Magento_Ui/js/lib/view/utils/async",
        "Magento_Checkout/js/model/quote",
        "Magento_Ui/js/form/element/select",
        "Magento_Checkout/js/model/shipping-rates-validator",
        "Magento_Checkout/js/model/shipping-rates-validation-rules",
        "Montonio_Shipping/js/model/shipping-rates-validator/validator",
        "Montonio_Shipping/js/model/shipping-rates-validation-rules/rules",
        "Montonio_Shipping/js/lib/select2"
    ],
    function (
        uiComponent,
        uiRegistry,
        ko,
        _,
        jQuery,
        $t,
        wrapper,
        domObserver,
        async,
        quote,
        select,
        defaultShippingRatesValidator,
        defaultShippingRatesValidationRules,
        shippingRatesValidator,
        shippingRatesValidationRules,
        select2
    ) {
        "use strict";

        defaultShippingRatesValidator.registerValidator("montonio-shipping", shippingRatesValidator);
        defaultShippingRatesValidationRules.registerRules("montonio-shipping", shippingRatesValidationRules);

        return uiComponent.extend({

            initialize: function () {
                this._super();
                this.current = false;
                this.subscribers = {};
                this.initDropdowns(this.points);
                this.initSubscriber();
                this.initValidation();
            },

            /**
             *
             * @param locations
             */
            initDropdowns: function (locations) {

                //if quote is new
                //if we have stored values
                //we can empty
             
                _.each(locations, function (loc) {

                    domObserver.get(
                        'input[value="montonio_shipping_' + loc + '"]',
                        function (elem) {
                            jQuery('<tr id="sdrop_' + loc + '" class="sdrop" style="display:none">' +
                                '<td colspan="4">' +
                                '<!-- ko foreach: getRegion(\'' + loc + '\') -->\n' +
                                '<!-- ko template: getTemplate() --><!-- /ko -->\n' +
                                '<!--/ko-->' +
                                '</td></tr>').insertAfter(jQuery(elem).parents('tr').first()
                            );

                            if (
                                elem.value.indexOf('montonio_shipping', 0) >= 0 &&
                                elem.value.indexOf('courier', 0) === -1
                            ) {
                                let sdropId = '#sdrop_' + loc;
                                let sdrop = jQuery(sdropId);
                                ko.applyBindings(this, sdrop[0]);
                                //detect when the node is bound
                                domObserver.get(
                                    sdropId + ' div.control select',
                                    function (select) {
                                        jQuery(select).select2();

                                        if(elem.checked) {
                                            sdrop.first().show();
                                            //setToAddress
                                            this.findAndSetAddress(loc);
                                        }

                                        //subscribe to observables
                                        let path = "checkout.steps.shipping-step.step-config.shipping-rates-validation.montonio_shipping.";
                                        let suffix = loc + "." + loc;
                                        uiRegistry.async(path + suffix)(
                                            function(drop) {
                                                if(_.isUndefined(this.subscribers[loc])) {
                                                    this.subscribers[loc] = drop.value.subscribe(
                                                        function(value) {
                                                            this.setToAddress(value);
                                                        }.bind(this),
                                                        true,
                                                        'change'
                                                    );
                                                }
                                            }.bind(this)
                                        );

                                        let method = quote.shippingMethod();
                                        if (!_.isNull(method)) {
                                            this.current = method.method_code;
                                        }

                                    }.bind(this)
                                );
                            }
                        }.bind(this)
                    );
                }.bind(this));
            },

            initSubscriber: function () {
                quote.shippingMethod.subscribe(
                    function (method) {

                        //not selected yet or unselected
                        if (
                            _.isNull(method) ||
                            _.isNull(method.carrier_code) ||
                            _.isNull(method.method_code)
                        ) {
                            return;
                        }

                        // if our something is selected, hide previous
                        let methodCode = method.method_code;
                        if (
                            this.current &&
                            this.current !== methodCode
                        ) {
                            jQuery("#sdrop_" + this.current).first().hide();
                        }

                        if (
                            !_.isNull(method) &&
                            !_.isNull(method.carrier_code) &&
                            method.carrier_code === "montonio_shipping" &&
                            methodCode.indexOf('courier', 0) === -1
                        ) {
                            //if ours show dropdown
                            jQuery("#sdrop_" + methodCode).first().show();
                            //setToAddress
                            this.findAndSetAddress(methodCode);
                            this.current = methodCode;
                        } else {
                            this.setToAddress(undefined);
                        }
                    }.bind(this),
                    true, 'change');
            },

            initValidation: function() {
                uiRegistry.async("checkout.steps.shipping-step.shippingAddress")(
                    function (shippingAddress) {
                        shippingAddress.validateShippingInformation = wrapper.wrap(
                            shippingAddress.validateShippingInformation,
                            function(originalMethod){
                                var isValid = true;
                                let method = quote.shippingMethod();
                                if (
                                    !_.isNull(method) ||
                                    !_.isUndefined(method.carrier_code) ||
                                    !_.isUndefined(method.method_code) ||
                                    !_.isUndefined(method.method_title)
                                ) {
                                    if (
                                        method.carrier_code === "montonio_shipping"
                                    ) {
                                        uiRegistry.get(
                                            {'dataScope': "montonio_shipping." + method.method_code},
                                            function(drop) {
                                                isValid =  drop.validate().valid;
                                            }.bind(this)
                                        );
                                    }

                                    if(!isValid) {
                                        this.errorValidationMessage(
                                            $t("Please pick the location for shipping method %s .").replace('%s', method.method_title)
                                        );
                                        return false;
                                    }
                                }

                                return originalMethod();
                            }
                        );
                    }.bind(this)
                );
            },

            setToAddress: function ( value) {
                let address = quote.shippingAddress();
                if(!_.isNull(address)) {
                    jQuery.extend(true , address , {'extension_attributes': {'montonio_shipping_point': value}});
                    if(_.isUndefined(value)) {
                        delete address.extension_attributes.montonio_shipping_point;
                        if(_.isEmpty(address.extension_attributes)) {
                            delete address.extension_attributes;
                        }
                    }
                }
                address = quote.billingAddress();
                if(!_.isNull(address)) {
                    jQuery.extend(true , address , {'extension_attributes': {'montonio_shipping_point': value}});
                    if(_.isUndefined(value)) {
                        delete address.extension_attributes.montonio_shipping_point;
                        if(_.isEmpty(address.extension_attributes)) {
                            delete address.extension_attributes;
                        }
                    }
                }
            },

            findAndSetAddress: function (methodCode) {
                let path = "checkout.steps.shipping-step.step-config.shipping-rates-validation.montonio_shipping.";
                let suffix = methodCode + "." + methodCode;
                uiRegistry.async(path + suffix)(
                    function (drop) {
                        this.setToAddress(drop.value());
                    }.bind(this)
                );
            }

        });
    }
);